=== modified file 'plainbox-gui/gui-engine/gui-engine.cpp'
--- plainbox-gui/gui-engine/gui-engine.cpp	2013-07-04 17:00:33 +0000
+++ plainbox-gui/gui-engine/gui-engine.cpp	2013-07-24 08:49:46 +0000
@@ -23,6 +23,176 @@
 #include <QtDBus/QtDBus>
 #include <QDebug>
 
+// Plainbox tree node class
+PBTreeNode::PBTreeNode()
+{
+    parent = NULL;
+    object_path.path().clear();
+    managed_objects.clear();
+    children.clear();
+    introspection = NULL;
+    xmlstring.clear();
+    interfaces.clear();
+}
+/* Add a new PBTreeNode based upon the supplied DBus object_path
+ */
+PBTreeNode* PBTreeNode::AddNode(PBTreeNode* parentNode, \
+                                const QDBusObjectPath &object_path)
+{
+    PBTreeNode* pbtn = NULL;
+
+    // special case for the root node
+    if(parentNode->object_path.path().isNull()) {
+        // We ARE the parentNode this time
+        pbtn = parentNode;
+    }
+    else {
+        pbtn = new PBTreeNode();
+    }
+
+    pbtn->object_path = object_path;
+    pbtn->parent=parentNode;
+
+    // The introspected string describing this object
+    const QString intro_xml = GetIntrospectXml(object_path);
+
+    pbtn->introspection=new QDomDocument(intro_xml);
+    pbtn->xmlstring = intro_xml;
+
+    /* We fill in all the children.
+     *
+     * We do this by creating a new child node for each node in intro_xml,
+     * and then introspecting these child nodes until we get nothing more
+     */
+    QDomDocument doc;
+    doc.setContent(intro_xml);
+    QDomElement xmlnode=doc.documentElement();
+    QDomElement child=xmlnode.firstChildElement();
+    while(!child.isNull()) {
+
+        // Is this a node?
+        if (child.tagName() == "node") {
+            // Yes, so we should introspect that as well
+            QString child_path;
+
+            if (object_path.path() == "/") {
+                child_path = object_path.path() + child.attribute("name");
+            } else {
+                child_path = object_path.path() + "/" + child.attribute("name");
+            }
+
+            QDBusObjectPath child_object_path(child_path);
+
+            PBTreeNode* node = AddNode(pbtn,child_object_path);
+            if (node) {
+                pbtn->children.append(node);
+            }
+        }
+
+        // Is this an interface?
+        if (child.tagName() == "interface") {
+            QString iface_name = child.attribute("name");
+
+            // we dont need properties from freedesktop interfaces
+            if (iface_name != ofDIntrospectableName && \
+                    iface_name != ofDPropertiesName) {
+
+                QVariantMap properties;
+
+                properties = GetObjectProperties(object_path, iface_name);
+
+                if (!properties.empty()) {
+                    PBObjectInterface *iface = \
+                            new PBObjectInterface(iface_name,properties);
+
+                    pbtn->interfaces.append(iface);
+                }
+            }
+        }
+        child = child.nextSiblingElement();
+    }
+
+    return pbtn;
+}
+const QString PBTreeNode::GetIntrospectXml(const QDBusObjectPath &object_path)
+{
+    // Connect to the introspectable interface
+    QDBusInterface iface(PBBusName, \
+                         object_path.path(), \
+                         ofDIntrospectableName, \
+                         QDBusConnection::sessionBus());
+    if (!iface.isValid()) {
+        qDebug("Could not connect to introspectable interface");
+        return NULL;
+    }
+
+    // Lets see what we have - introspect this first
+    QDBusMessage reply = iface.call("Introspect");
+    if (reply.type() != QDBusMessage::ReplyMessage) {
+        qDebug("Could not introspect this object");
+        return NULL;
+    }
+
+    QList<QVariant> args = reply.arguments();
+
+    QList<QVariant>::iterator iter = args.begin();
+
+    QVariant variant = *iter;
+
+    // The introspected string describing this object
+    const QString intro_xml = variant.value<QString>();
+
+    return intro_xml;
+}
+QVariantMap PBTreeNode::GetObjectProperties(const QDBusObjectPath &object_path, \
+                                            const QString interface)
+{
+    QVariantMap properties;
+
+    // Connect to the freedesktop Properties interface
+    QDBusInterface iface(PBBusName, \
+                         object_path.path(), \
+                         "org.freedesktop.DBus.Properties", \
+                         QDBusConnection::sessionBus());
+    if (!iface.isValid()) {
+        qDebug("Could not connect to properties interface");
+        return properties;
+    }
+
+    // GetAll properties
+    QDBusMessage reply = iface.call("GetAll",interface);
+    if (reply.type() != QDBusMessage::ReplyMessage) {
+        qDebug("Could not get the properties");
+        return properties;
+    }
+
+    QList<QVariant> args = reply.arguments();
+
+    if (args.empty()) {
+        return properties;
+    }
+
+    QList<QVariant>::iterator p = args.begin();
+
+    QVariant variant = *p;
+
+    const QDBusArgument qda = variant.value<QDBusArgument>();
+
+    qda >> properties;
+
+    return properties;
+}
+
+PBTreeNode::~PBTreeNode()
+{
+    if (children.count()) {
+
+        QList<PBTreeNode*>::iterator p;
+
+        children.erase(children.begin(),children.end());
+    }
+}
+
 void GuiEnginePlugin::registerTypes(const char *uri)
 {
     // register our classes
@@ -31,39 +201,452 @@
 
 GuiEngine::GuiEngine( QObject*parent ) : QObject(parent)
 {
-
-}
-
-// Dummy_CallPlainbox_Exit is for prototyping purposes and will eventually be removed
-void GuiEngine::Dummy_CallPlainbox_Exit()
-{
-    // Connect to Dbus Session Bus
-    if (!QDBusConnection::sessionBus().isConnected()) {
-        fprintf(stderr, "Cannot connect to the D-Bus session bus.\n");
-        return;
-    }
-
-    // Hard-coded names for Plainbox session, service and function
-    QDBusInterface iface("com.canonical.certification.PlainBox", \
-    	"/plainbox/service", "com.canonical.certification.PlainBox", \
-    	QDBusConnection::sessionBus());
-    if (iface.isValid()) {
-        QDBusReply<QString> reply = iface.call("Exit");
-        if (reply.isValid()) {
-            printf("Reply was: %s\n", qPrintable(reply.value()));
-            return;
-        } else {
-            fprintf(stderr,"Invalid reply, but not important for now. \
-	            This code shows where to collect results from DBus calls");
-        }
-        return;
-    }
-
-    fprintf(stderr, "%s\n", \
-    	qPrintable(QDBusConnection::sessionBus().lastError().message()));
-}
-
-// End of file - gui-engine.cpp
-
-
-
+    qDebug("GuiEngine::GuiEngine");
+
+    enginestate = UNINITIALISED;
+    pb_objects=NULL;
+    valid_pb_objects = false;
+
+    qDebug("GuiEngine::GuiEngine - Done");
+}
+
+bool GuiEngine::Initialise(void)
+{
+    qDebug("GuiEngine::Initialise");
+
+    // Only do this once
+    if (enginestate==UNINITIALISED) {
+
+        qDebug("GuiEngine - Initialising");
+
+        // Connect to Dbus Session Bus
+        if (!QDBusConnection::sessionBus().isConnected()) {
+            qDebug("Cannot connect to the D-Bus session bus.\n");
+
+            return false;
+        }
+
+        // Register our custom types for unpacking the object tree
+        qDBusRegisterMetaType<om_smalldict>();
+        qDBusRegisterMetaType<om_innerdict>();
+        qDBusRegisterMetaType<om_outerdict>();
+
+        // Obtain the initial tree of Plainbox objects, starting at the root "/"
+        pb_objects = new PBTreeNode();
+        pb_objects->AddNode(pb_objects, QDBusObjectPath("/"));
+        if (!pb_objects) {
+            qDebug("Failed to get Plainbox Objects");
+
+            return false;
+        }
+
+        // Connect our change receivers
+        QDBusConnection bus = QDBusConnection ::sessionBus();
+
+        if (!bus.connect(PBBusName, \
+                         PBObjectPathName, \
+                         ofDObjectManagerName,\
+                         "InterfacesAdded",\
+                         this,\
+                         SLOT(InterfacesAdded(QDBusMessage)))) {
+
+            qDebug("Failed to connect slot for InterfacesAdded events");
+
+            return false;
+        }
+
+        if (!bus.connect(PBBusName,\
+                         PBObjectPathName,\
+                         ofDObjectManagerName,\
+                         "InterfacesRemoved",\
+                         this,\
+                         SLOT(InterfacesRemoved(QDBusMessage)))) {
+
+            qDebug("Failed to connect slot for InterfacesRemoved events");
+
+            return false;
+        }
+
+        enginestate = READY;
+    }
+
+    qDebug("GuiEngine::Initialise() - Done");
+
+    return true;
+}
+
+bool GuiEngine::Shutdown(void)
+{
+    qDebug("GuiEngine::Shutdown()");
+
+    // Were we initialised?
+    if (enginestate==UNINITIALISED) {
+        qDebug("Plainbox GUI Engine not initialised");
+        return false;
+    }
+
+    QDBusInterface iface(PBBusName, \
+                         PBObjectPathName, \
+                         PBInterfaceName, \
+                         QDBusConnection::sessionBus());
+    if (!iface.isValid()) {
+        qDebug("Cant get Plainbox Service interface");
+        return false;
+    }
+
+    QDBusMessage reply = iface.call("Exit");
+    if (reply.type() != QDBusMessage::ReplyMessage) {
+
+        qDebug() << "Failed to exit Plainbox" << reply.errorMessage();
+
+        return false;
+    }
+
+    enginestate=UNINITIALISED;
+
+    qDebug("GuiEngine::Shutdown() - Done");
+
+    return true;
+}
+// DBus-QT Demarshalling helper functions
+const QDBusArgument &operator>>(const QDBusArgument &argument, \
+                                om_smalldict &smalldict)
+{
+    argument.beginMap();
+    smalldict.clear();
+
+    while(!argument.atEnd())
+    {
+        argument.beginMapEntry();
+
+        QString property;
+        QDBusVariant variant;
+
+        argument >> property >> variant;
+
+        qDebug() << "string" \
+                 << property \
+                 << "variant: " \
+                 << variant.variant().toString();
+
+        smalldict.insert(property,variant);
+
+        argument.endMapEntry();
+    }
+
+    argument.endMap();
+
+    return argument;
+}
+
+const QDBusArgument &operator>>(const QDBusArgument &argument, \
+                                om_innerdict &innerdict)
+{
+    argument.beginMap();
+    innerdict.clear();
+
+    while(!argument.atEnd())
+    {
+        argument.beginMapEntry();
+
+        QString interface;
+        om_smalldict sd;
+
+        argument >> interface >> sd;
+
+        qDebug() << "Interface: " << interface;
+
+        innerdict.insert(interface,sd);
+
+        argument.endMapEntry();
+    }
+
+    argument.endMap();
+
+    return argument;
+}
+
+const QDBusArgument &operator>>(const QDBusArgument &argument, \
+                                om_outerdict &outerdict)
+{
+    argument.beginMap();
+    outerdict.clear();
+
+    while(!argument.atEnd())
+    {
+        argument.beginMapEntry();
+
+        QDBusObjectPath object_path;
+        om_innerdict innerdict;
+
+        argument >> object_path >> innerdict;
+
+        qDebug() << "ObjectPath" \
+                 << object_path.path();
+
+        argument.endMapEntry();
+
+        outerdict.insert(object_path,innerdict);
+    }
+
+    argument.endMap();
+
+    return argument;
+}
+const PBTreeNode* GuiEngine::GetPlainBoxObjects()
+{
+    return pb_objects;
+}
+
+const PBTreeNode* GuiEngine::GetRootJobsNode(const PBTreeNode* node)
+{
+    // Are we there yet
+    if (node->object_path.path() == "/plainbox/job") {
+        return node;
+    }
+
+    PBTreeNode* jobnode = NULL;
+
+    QList<PBTreeNode*>::const_iterator iter= node->children.begin();
+
+    while(!jobnode && iter !=  node->children.end()) {
+
+        PBTreeNode* child = *iter;
+
+        jobnode = const_cast<PBTreeNode*>(GetRootJobsNode(child));
+        if (jobnode) {
+            return jobnode;
+        }
+
+        iter++;
+    }
+
+    return NULL;
+}
+const PBTreeNode* GuiEngine::GetRootWhiteListNode(const PBTreeNode* node)
+{
+    // Are we there yet
+    if (node->object_path.path() == "/plainbox/whitelist") {
+        return node;
+    }
+
+    PBTreeNode* whitenode = NULL;
+
+    QList<PBTreeNode*>::const_iterator iter= node->children.begin();
+
+    while(!whitenode && iter !=  node->children.end()) {
+
+        PBTreeNode* child = *iter;
+
+        whitenode = const_cast<PBTreeNode*>(GetRootWhiteListNode(child));
+        if (whitenode) {
+            return whitenode;
+        }
+
+        iter++;
+    }
+
+    return NULL;
+}
+// Work in progress
+void GuiEngine::InterfacesAdded(QDBusMessage msg)
+{
+    qDebug("GuiEngine::InterfacesAdded");
+
+    QList<QVariant> args = msg.arguments();
+
+    QList<QVariant>::iterator p = args.begin();
+
+    p = args.begin();
+
+    QVariant q = *p;
+
+    QDBusObjectPath opath = q.value<QDBusObjectPath>();
+
+    qDebug() << "objectpath: " << opath.path();
+
+    p++;
+
+    q = *p;
+
+    const QDBusArgument qda = q.value<QDBusArgument>();
+
+    om_innerdict my_id;
+
+    qda >> my_id;
+
+    // Need to update our tree of objects.
+
+    // todo
+
+    // Now tell the GUI to update its knowledge of the objects
+    emit UpdateGuiObjects();
+
+    qDebug("GuiEngine::InterfacesAdded - done");
+}
+
+// Work in progress
+void GuiEngine::InterfacesRemoved(QDBusMessage msg)
+{
+    qDebug("GuiEngine::InterfacesRemoved");
+
+    // Need to update our tree of objects.
+
+    qDebug() << "Signature is: " << msg.signature();
+
+    QList<QVariant> args = msg.arguments();
+
+    qDebug("Reply arguments: %d",args.count());
+
+    QList<QVariant>::iterator p = args.begin();
+
+    p = args.begin();
+
+    QVariant q = *p;
+
+    QDBusObjectPath opath = q.value<QDBusObjectPath>();
+
+    qDebug() << "opath: " << opath.path();
+
+    p++;
+
+    q = *p;
+
+    const QDBusArgument qda = q.value<QDBusArgument>();
+
+    // Need to update our tree of objects.
+
+    // todo
+
+    // Now tell the GUI to update its knowledge of the objects
+    emit UpdateGuiObjects();
+
+    qDebug("GuiEngine::InterfacesRemoved - done");
+}
+QList<PBTreeNode*> GuiEngine::GetJobNodes(void)
+{
+    qDebug("GuiEngine::GetJobNodes()");
+
+    QList<PBTreeNode*> jobnodes;
+
+    PBTreeNode* jobnode = const_cast<PBTreeNode*>(GetRootJobsNode(GetPlainBoxObjects()));
+    if (!jobnode) {
+        return jobnodes;
+    }
+
+    QList<PBTreeNode*>::const_iterator iter =jobnode->children.begin();
+
+    while(iter != jobnode->children.end()) {
+
+        PBTreeNode* child = *iter;
+
+         jobnodes.append(child);
+
+         iter++;
+    }
+
+    qDebug("GuiEngine::GetJobNodes() - done");
+
+    return jobnodes;
+}
+QList<PBTreeNode*> GuiEngine::GetWhiteListNodes(void)
+{
+    qDebug("GuiEngine::GetWhiteListNodes()");
+
+    QList<PBTreeNode*> whitelistnodes;
+
+    PBTreeNode* whitelistnode = const_cast<PBTreeNode*>(GetRootWhiteListNode(GetPlainBoxObjects()));
+    if (!whitelistnode) {
+        return whitelistnodes;
+    }
+
+    QList<PBTreeNode*>::const_iterator iter = whitelistnode->children.begin();
+
+    while(iter != whitelistnode->children.end()) {
+
+        PBTreeNode* child = *iter;
+
+         whitelistnodes.append(child);
+
+         iter++;
+    }
+
+    qDebug("GuiEngine::GetWhiteListNodes() - done");
+
+    return whitelistnodes;
+}
+QMap<QDBusObjectPath,QString> GuiEngine::GetWhiteListPathsAndNames(void)
+{
+    QMap<QDBusObjectPath,QString> paths_and_names;
+
+    PBTreeNode* whitenode = \
+            const_cast<PBTreeNode*>(GetRootWhiteListNode(GetPlainBoxObjects()));
+    if (!whitenode) {
+        return paths_and_names;
+    }
+
+    // loop around all the children
+    QList<PBTreeNode*>::const_iterator iter =whitenode->children.begin();
+
+    bool initialised = ! whitelist.empty();
+
+    while(iter != whitenode->children.end()) {
+
+        PBTreeNode* child = *iter;
+
+        QString opath = child->object_path.path();
+
+        // Connect to the introspectable interface
+        QDBusInterface introspect_iface(PBBusName, \
+                             opath, \
+                             "org.freedesktop.DBus.Properties", \
+                             QDBusConnection::sessionBus());
+
+        if (introspect_iface.isValid()) {
+            QDBusReply<QVariant> reply  = introspect_iface.call("Get", \
+                       "com.canonical.certification.PlainBox.WhiteList1", \
+                       "name");
+
+            QVariant var(reply);
+
+            QString name(var.toString());
+
+            qDebug() << name;
+
+            paths_and_names.insert(child->object_path,name);
+
+            // First time round, fill in our whitelist member
+            if (!initialised) {
+                whitelist.insert(child->object_path,true);
+            }
+        }
+
+        iter++;
+    }
+
+    return paths_and_names;
+}
+
+// temporary helper
+void GuiEngine::SetWhiteList(const QDBusObjectPath opath, const bool check)
+{
+    whitelist.remove(opath);
+
+    whitelist.insert(opath,check);
+}
+
+// temporary helper
+void GuiEngine::dump_whitelist_selection(void)
+{
+    // loop around all the children
+    QMap<QDBusObjectPath,bool>::const_iterator iter = whitelist.begin();
+
+    while(iter != whitelist.end() ) {
+
+        bool yes = iter.value();
+
+        qDebug() << iter.key().path() << (yes ? "Yes" : "No");
+
+        iter++;
+    }
+}

=== modified file 'plainbox-gui/gui-engine/gui-engine.h'
--- plainbox-gui/gui-engine/gui-engine.h	2013-07-04 17:00:33 +0000
+++ plainbox-gui/gui-engine/gui-engine.h	2013-07-24 08:49:46 +0000
@@ -18,34 +18,187 @@
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
- 
+
  #ifndef PLAINBOX_GUI_H
  #define PLAINBOX_GUI_H
- 
+
 #include <QtQml/qqml.h>
 #include <QtQml/QQmlExtensionPlugin>
+#include <QtDBus/QtDBus>
+#include <QtXml/QDomDocument>
 
 class GuiEnginePlugin : public QQmlExtensionPlugin
 {
     Q_OBJECT
     Q_PLUGIN_METADATA(IID "guiengine")
-	
+
 public:
 
     // inherited from QQmlExtensionPlugin
     void registerTypes(const char *uri);
 };
- 
-
+
+/* Decoding for the org.freedesktop.Dbus.ObjectManager
+ *
+ * Structs useful to the GUI for extraction and display purposes
+ *
+ * Ultimately we must decode this DBus Signature
+ * "a{oa{sa{sv}}}
+ *
+ * So, starting with the inner most, we can say that:
+ *   a{sv} = QString, QDBusVariant = om_smalldict
+ *
+ * Then, we will have
+ *
+ * a{sa{sv}} = QMap<QString,QVariantMap> = om_innerdict
+ *
+ * Finally, we want this:
+ *
+ * a{oa{sa{sv}}} = QMap<QString,innerdict> = outerdict
+ */
+typedef QMap<QString,QDBusVariant> om_smalldict;
+typedef QMap<QString,om_smalldict> om_innerdict;
+typedef QMap<QDBusObjectPath,om_innerdict> om_outerdict;
+
+// now register these metatypes
+Q_DECLARE_METATYPE(om_smalldict);
+Q_DECLARE_METATYPE(om_innerdict);
+Q_DECLARE_METATYPE(om_outerdict);
+
+/* Dbus specification standard interface names - they dont appear to be defined
+ * by Qt DBus system.
+ */
+
+static const QString ofDObjectManagerName("org.freedesktop.DBus.ObjectManager");
+static const QString ofDIntrospectableName("org.freedesktop.DBus.Introspectable");
+static const QString ofDPropertiesName("org.freedesktop.DBus.Properties");
+
+/* The names for Plainbox top-level DBus structures.
+ */
+static const QString PBBusName("com.canonical.certification.PlainBox");
+static const QString PBObjectPathName("/plainbox/service1");
+static const QString PBInterfaceName("com.canonical.certification.PlainBox.Service1");
+
+/* Represents a DBus Object Interface. We only need the properties as the
+ * methods are only of a few fixed types we already know.
+ */
+class PBObjectInterface
+{
+public:
+    PBObjectInterface(QString interface, QVariantMap properties)
+        : interface(interface), properties(properties)
+    {
+
+    };
+
+    ~PBObjectInterface();
+
+public:
+    QString interface;
+    // Methods - not clear what if anything needs to be stored for these?
+    QVariantMap properties;
+};
+
+// Well-known Plainbox/Checkbox interfaces
+static const QString PlainboxJobDefinition1("com.canonical.certification.PlainBox.JobDefinition1");
+static const QString CheckBoxJobDefinition1("com.canonical.certification.CheckBox.JobDefinition1");
+
+/* This enables us to build a tree of Plainbox objects. We do this
+ * by recursively introspecting the tree over DBus
+ */
+class PBTreeNode
+{
+public:
+    PBTreeNode();
+
+    ~PBTreeNode();
+
+    PBTreeNode* AddNode(PBTreeNode* parentNode, \
+                        const QDBusObjectPath &object_path);
+
+private:
+    QVariantMap GetObjectProperties(const QDBusObjectPath &object_path, \
+                                    const QString interface);
+    const QString GetIntrospectXml(const QDBusObjectPath &object_path);
+
+public:
+    PBTreeNode *parent;
+
+    QDBusObjectPath object_path;
+    om_outerdict managed_objects;
+    QList<PBTreeNode*> children;
+    QDomDocument* introspection;
+    QString xmlstring;
+
+    QList<PBObjectInterface*> interfaces;
+};
+
+/* This class embodies the wrapper which can call Plainbox APIs over D-Bus
+ *
+ * Its intended clients are Test Suite selection, test selection etc.
+ *
+ * There should only be one instance of this class
+ */
 class GuiEngine : public QObject
 {
     Q_OBJECT
 
-    public:
+    private:
+        enum EngineState {
+        UNINITIALISED,
+        READY
+        };
+
+public:
         GuiEngine( QObject*parent = 0);
-    public slots:
-        // Dummy_CallPlainbox_Exit is for prototyping purposes and will eventually be removed
-        void Dummy_CallPlainbox_Exit();
+
+public slots:
+        // Manage GuiEngine lifetime
+        bool Initialise(void);
+        bool Shutdown(void);
+
+        // update object tree based on callbacks from plainbox/dbus
+        void InterfacesAdded(QDBusMessage msg);
+        void InterfacesRemoved(QDBusMessage msg);
+
+        // Handle to the tree of PlainBox objects
+        const PBTreeNode* GetPlainBoxObjects(void);
+
+        // temporary
+        void dump_whitelist_selection(void);
+signals:
+        // Instruct the GUI to update itself
+        void UpdateGuiObjects(void);
+
+public:
+        // Temporary public functions
+        const PBTreeNode* GetRootJobsNode(const PBTreeNode *node);
+        const PBTreeNode* GetRootWhiteListNode(const PBTreeNode *node);
+
+        QList<PBTreeNode*> GetJobNodes(void);
+        QList<PBTreeNode*> GetWhiteListNodes(void);
+
+        // Returns whitelist object path, whitelist name
+        QMap<QDBusObjectPath,QString> GetWhiteListPathsAndNames(void);
+
+        void SetWhiteList(const QDBusObjectPath opath, const bool check);
+
+private:
+        EngineState enginestate;
+
+        // Contains our Tree of Plainbox objects (Methods, results, tests etc)
+        PBTreeNode* pb_objects;
+
+        // Have we got valid tree of PlainBox objects?
+        bool valid_pb_objects;
+
+        // These may go later, but are helpful for now
+
+        // A list of the selected whitelists from the user
+        QMap<QDBusObjectPath, bool> whitelist;
+
+        // A user-selected list of tests. We store the object path
+        QMap<QDBusObjectPath, bool> tests;
 };
- 
+
  #endif

=== modified file 'plainbox-gui/gui-engine/gui-engine.pro'
--- plainbox-gui/gui-engine/gui-engine.pro	2013-07-04 17:00:33 +0000
+++ plainbox-gui/gui-engine/gui-engine.pro	2013-07-24 08:49:46 +0000
@@ -23,7 +23,7 @@
 
 TEMPLATE = lib
 CONFIG += qt plugin
-QT +=qml dbus
+QT +=qml dbus xml
 
 TARGET = gui-engine
 

=== modified file 'plainbox-gui/plainbox-gui.pro'
--- plainbox-gui/plainbox-gui.pro	2013-07-04 17:00:33 +0000
+++ plainbox-gui/plainbox-gui.pro	2013-07-24 08:49:46 +0000
@@ -19,5 +19,5 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 TEMPLATE = subdirs
-SUBDIRS = gui-engine gui-ihv
+SUBDIRS = gui-engine gui-ihv test-gui-engine
 CONFIG = ordered

=== added directory 'plainbox-gui/test-gui-engine'
=== added file 'plainbox-gui/test-gui-engine/README.rst'
--- plainbox-gui/test-gui-engine/README.rst	1970-01-01 00:00:00 +0000
+++ plainbox-gui/test-gui-engine/README.rst	2013-07-24 08:49:46 +0000
@@ -0,0 +1,8 @@
+test-gui-engine
+===============
+
+This folder contains simple test code for Qt/C++ to D-Bus engine plugin.
+
+
+
+

=== added file 'plainbox-gui/test-gui-engine/test-gui-engine.cpp'
--- plainbox-gui/test-gui-engine/test-gui-engine.cpp	1970-01-01 00:00:00 +0000
+++ plainbox-gui/test-gui-engine/test-gui-engine.cpp	2013-07-24 08:49:46 +0000
@@ -0,0 +1,35 @@
+/*
+ * This file is part of plainbox-gui
+ *
+ * Copyright 2013 Canonical Ltd.
+ *
+ * Authors:
+ * - Andrew Haigh <andrew.haigh@cellsoftware.co.uk>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; version 3.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ */
+
+#include "test-gui-engine.h"
+
+void TestGuiEngine::TestInitialise()
+{
+    QVERIFY(Initialise());
+}
+
+void TestGuiEngine::TestShutdown()
+{
+    QVERIFY(Shutdown());
+}
+
+QTEST_MAIN(TestGuiEngine)
+

=== added file 'plainbox-gui/test-gui-engine/test-gui-engine.h'
--- plainbox-gui/test-gui-engine/test-gui-engine.h	1970-01-01 00:00:00 +0000
+++ plainbox-gui/test-gui-engine/test-gui-engine.h	2013-07-24 08:49:46 +0000
@@ -0,0 +1,32 @@
+/*
+ * This file is part of plainbox-gui
+ *
+ * Copyright 2013 Canonical Ltd.
+ *
+ * Authors:
+ * - Andrew Haigh <andrew.haigh@cellsoftware.co.uk>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; version 3.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ */
+
+#include <QtTest/QtTest>
+
+#include "../gui-engine/gui-engine.h"
+
+ class TestGuiEngine: public GuiEngine
+ {
+     Q_OBJECT
+ private slots:
+     void TestInitialise();
+     void TestShutdown();
+ };

=== added file 'plainbox-gui/test-gui-engine/test-gui-engine.pro'
--- plainbox-gui/test-gui-engine/test-gui-engine.pro	1970-01-01 00:00:00 +0000
+++ plainbox-gui/test-gui-engine/test-gui-engine.pro	2013-07-24 08:49:46 +0000
@@ -0,0 +1,30 @@
+# This file is part of plainbox-gui
+#
+# Copyright 2013 Canonical Ltd.
+#
+# Authors:
+# - Andrew Haigh <andrew.haigh@cellsoftware.co.uk>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; version 3.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+#
+# test-gui-engine.pro
+#
+# Hand-made pro file to create the test executable for gui-engine plugin
+
+QT  += testlib dbus qml xml
+
+HEADERS += test-gui-engine.h ../gui-engine/gui-engine.h
+
+SOURCES += test-gui-engine.cpp  ../gui-engine/gui-engine.cpp
+
+

